@@ -2,10 +2,14 @@ import argparse
 
 from devassistant.cli import devassistant_argparse
 from devassistant import settings
+from devassistant import utils
+
 
 class ArgparseGenerator(object):
     subparsers_str = 'subassistants'
     subparsers_desc = '''Following subassistants will help you with setting up your project.'''
+    subparsers_no_avail = 'No subasistants are available. Please install ' +\
+        '"assistants" package first.'
     subactions_str = 'subactions'
     subactions_desc = 'This action has following subactions.'
 
@@ -34,13 +38,13 @@ class ArgparseGenerator(object):
 
         if cur_subas or actions:
             # then add the subassistants as arguments
-            subparsers = parser.add_subparsers(dest=settings.SUBASSISTANT_N_STRING.format('0'))
-            # from Python 3.3, subparsers are optional by default => make them required
-            subparsers.required=True
+            subparsers = cls._add_subparsers_required(parser,
+                dest=settings.SUBASSISTANT_N_STRING.format('0'))
             for subas in sorted(cur_subas, key=lambda x: x[0].name):
-                cls.add_subassistants_to(subparsers, subas, level=1)
+                for alias in [subas[0].name] + getattr(subas[0], 'aliases', []):
+                    cls.add_subassistants_to(subparsers, subas, level=1, alias=alias)
 
-            for action, subactions in sorted(actions.items(), key=lambda x:x[0].name):
+            for action, subactions in sorted(actions.items(), key=lambda x: x[0].name):
                 cls.add_action_to(subparsers, action, subactions, level=1)
 
         return parser
@@ -53,19 +57,10 @@ class ArgparseGenerator(object):
                             action='store_true',
                             dest='da_debug',
                             default=False)
-        # TODO: should gui also get --no-cache?
-        # This really only stores the True/False value. We need to set
-        # current_run.USE_CACHE before we create the parser, but for creating
-        # the parser, we need to load assistants. That means we set
-        # current_run.USE_CACHE in cli_runner according to sys.argv.
-        parser.add_argument('--no-cache',
-                            help='Don\'t use assistants cache (useful for debugging).',
-                            action='store_true',
-                            dest='da_no_cache',
-                            default=False)
+        utils.add_no_cache_argument(parser)
 
     @classmethod
-    def add_subassistants_to(cls, parser, assistant_tuple, level):
+    def add_subassistants_to(cls, parser, assistant_tuple, level, alias=None):
         """Adds assistant from given part of assistant tree and all its subassistants to
         a given argument parser.
 
@@ -74,18 +69,25 @@ class ArgparseGenerator(object):
             assistant_tuple: part of assistant tree (see generate_argument_parser doc)
             level: level of subassistants that given assistant is at
         """
-        p = parser.add_parser(assistant_tuple[0].name,
+        name = alias or assistant_tuple[0].name
+        p = parser.add_parser(name,
                               description=assistant_tuple[0].description,
                               argument_default=argparse.SUPPRESS)
         for arg in assistant_tuple[0].args:
             arg.add_argument_to(p)
 
         if len(assistant_tuple[1]) > 0:
-            subparsers = p.add_subparsers(dest=settings.SUBASSISTANT_N_STRING.format(level),
-                                          title=cls.subparsers_str,
-                                          description=cls.subparsers_desc)
+            subparsers = cls._add_subparsers_required(p,
+                dest=settings.SUBASSISTANT_N_STRING.format(level),
+                title=cls.subparsers_str,
+                description=cls.subparsers_desc)
             for subas_tuple in sorted(assistant_tuple[1], key=lambda x: x[0].name):
                 cls.add_subassistants_to(subparsers, subas_tuple, level + 1)
+        elif level == 1:
+            subparsers = cls._add_subparsers_required(p,
+                dest=settings.SUBASSISTANT_N_STRING.format(level),
+                title=cls.subparsers_str,
+                description=cls.subparsers_no_avail)
 
     @classmethod
     def add_action_to(cls, parser, action, subactions, level):
@@ -103,8 +105,16 @@ class ArgparseGenerator(object):
             arg.add_argument_to(p)
 
         if subactions:
-            subparsers = p.add_subparsers(dest=settings.SUBASSISTANT_N_STRING.format(level),
-                                          title=cls.subactions_str,
-                                          description=cls.subactions_desc)
+            subparsers = cls._add_subparsers_required(p,
+                dest=settings.SUBASSISTANT_N_STRING.format(level),
+                title=cls.subactions_str,
+                description=cls.subactions_desc)
             for subact, subsubacts in sorted(subactions.items(), key=lambda x: x[0].name):
                 cls.add_action_to(subparsers, subact, subsubacts, level + 1)
+
+    @classmethod
+    def _add_subparsers_required(self, parser, **kwargs):
+        # from Python 3.3, subparsers are optional by default => make them required
+        subparsers = parser.add_subparsers(**kwargs)
+        subparsers.required = True
+        return subparsers
