@@ -1,6 +1,7 @@
 from __future__ import print_function
 
 import atexit
+import errno
 import getpass
 import logging
 import os
@@ -10,9 +11,9 @@ import sys
 
 import six
 
-from devassistant import current_run
 from devassistant import exceptions
 from devassistant.logger import logger
+from devassistant.settings import ROOT_EXECUTABLE
 
 
 class ClHelper(object):
@@ -26,7 +27,9 @@ class ClHelper(object):
                     log_level=logging.DEBUG,
                     ignore_sigint=False,
                     output_callback=None,
-                    as_user=None):
+                    as_user=None,
+                    log_secret=False,
+                    env=None):
         """Runs a command from string, e.g. "cp foo bar"
         Args:
             cmd_str: the command to run as string
@@ -35,6 +38,10 @@ class ClHelper(object):
             output_callback: function that gets called with every line of output as argument
             as_user: run as specified user (the best way to do this will be deduced by DA)
                 runs as current user if as_user == None
+            log_secret: if True, the command invocation will only be logged as
+                "LOGGING PREVENTED FOR SECURITY REASONS", no output will be logged
+            env: if not None, pass to subprocess as shell environment; else use
+                original DevAssistant environment
         """
         # run format processors on cmd_str
         for name, cmd_proc in cls.command_processors.items():
@@ -43,7 +50,7 @@ class ClHelper(object):
         # TODO: how to do cd with as_user?
         if as_user and not cmd_str.startswith('cd '):
             cmd_str = cls.format_for_another_user(cmd_str, as_user)
-        logger.log(log_level, cmd_str, extra={'event_type': 'cmd_call'})
+        cls.log(log_level, cmd_str, 'cmd_call', log_secret)
 
         if cmd_str.startswith('cd '):
             # special-case cd to behave like shell cd and stay in the directory
@@ -61,24 +68,32 @@ class ClHelper(object):
         stdout_pipe = subprocess.PIPE
         stderr_pipe = subprocess.STDOUT
         preexec_fn = cls.ignore_sigint if ignore_sigint else None
+        env = os.environ if env is None else env
         proc = subprocess.Popen(cmd_str,
                                 stdin=stdin_pipe,
                                 stdout=stdout_pipe,
                                 stderr=stderr_pipe,
                                 shell=True,
-                                preexec_fn=preexec_fn)
+                                preexec_fn=preexec_fn,
+                                env=env)
         # register process to cls.subprocesses
         cls.subprocesses[proc.pid] = proc
 
         stdout = []
         while proc.poll() is None:
-            output = proc.stdout.readline().decode('utf8')
-            if output:
-                output = output.strip()
-                stdout.append(output)
-                logger.log(log_level, output, extra={'event_type': 'cmd_out'})
-            if output_callback:
-                output_callback(output)
+            try:
+                output = proc.stdout.readline().decode('utf8')
+                if output:
+                    output = output.strip()
+                    stdout.append(output)
+                    cls.log(log_level, output, 'cmd_out', log_secret)
+                if output_callback:
+                    output_callback(output)
+            except IOError as e:
+                if e.errno == errno.EINTR:  # Interrupted system call in Python 2.6
+                    sys.stderr.write('Can\'t interrupt this process!\n')
+                else:
+                    raise e
 
         # remove process from cls.subprocesses
         cls.subprocesses.pop(proc.pid)
@@ -91,7 +106,7 @@ class ClHelper(object):
         # we want to log lines separately, not as one big chunk
         output_rest_lines = output_rest.splitlines()
         for i, l in enumerate(output_rest_lines):
-            logger.log(log_level, l, extra={'event_type': 'cmd_out'})
+            cls.log(log_level, l, 'cmd_out', log_secret)
             # add newline for every line - for last line, only add it if it was originally present
             if i != len(output_rest_lines) - 1 or output_rest.endswith('\n'):
                 l += '\n'
@@ -100,7 +115,7 @@ class ClHelper(object):
                 output_callback(l)
 
         # log return code always on debug level
-        logger.log(logging.DEBUG, proc.returncode, extra={'event_type': 'cmd_retcode'})
+        cls.log(logging.DEBUG, proc.returncode, 'cmd_retcode', log_secret)
         stdout = stdout.strip()
 
         if proc.returncode == 0:
@@ -117,11 +132,15 @@ class ClHelper(object):
         heredoc_firstline = ['pkexec']
         if as_user != 'root':
             heredoc_firstline.extend(['--user',  as_user])
-        heredoc_firstline.append('bash << {delim}'.format(delim=delimiter))
+        if os.path.isfile(ROOT_EXECUTABLE) and os.access(ROOT_EXECUTABLE, os.X_OK):
+            executable = ROOT_EXECUTABLE
+        else:
+            executable = 'bash'
+        heredoc_firstline.append('{exe} << {delim}'.format(exe=executable, delim=delimiter))
         cmd = '\n'.join([' '.join(heredoc_firstline),
                          cmd_str,
                          delimiter
-        ])
+                         ])
         return cmd
 
     @classmethod
@@ -134,6 +153,18 @@ class ClHelper(object):
             logger.info('Killing still running process {pid} ...'.format(pid=pid))
             proc.kill()
 
+    @classmethod
+    def log(cls, level, msg, event_type, secret):
+        extra = {'event_type': event_type}
+        if secret:
+            if event_type == 'cmd_call':
+                logger.log(level, 'LOGGING PREVENTED FOR SECURITY REASONS', extra=extra)
+            elif event_type == 'cmd_retcode':
+                logger.log(level, msg, extra=extra)
+            # no output logging for secret commands
+        else:
+            logger.log(level, msg, extra=extra)
+
 
 atexit.register(ClHelper.kill_subprocesses)
 
@@ -169,6 +200,8 @@ class DialogHelper(object):
     """This class is to be used in all places where user interaction is required. It will
     decide on its own which specific helper it is best to use in this place (CommandLine,
     Zenity, possibly other registered).
+    Note, that all methods that are supposed to actually interact with user should accept
+    "ui" as the first argument (the type of UI, e.g. "cli", "gui_gtk+")
     """
     helpers = {}
 
@@ -179,31 +212,37 @@ class DialogHelper(object):
         return helper
 
     @classmethod
-    def get_appropriate_helper(cls):
-        return cls.helpers[current_run.UI]
+    def get_appropriate_helper(cls, ui):
+        return cls.helpers[ui]
 
     @classmethod
-    def ask_for_password(cls, prompt='Provide your password:', **options):
+    def ask_for_password(cls, ui, prompt='Provide your password:', **options):
         """Returns the password typed by user as a string or None if user cancels the request
         (e.g. presses Ctrl + D on commandline or presses Cancel in GUI.
         """
         # optionally set title, that may be used by some helpers like zenity
-        return cls.get_appropriate_helper().ask_for_password(prompt,
-                                                             title=options.get('title', prompt))
+        return cls.get_appropriate_helper(ui).ask_for_password(prompt,
+                                                               title=options.get('title', prompt))
 
     @classmethod
-    def ask_for_confirm_with_message(cls, prompt='Do you agree?', message='', **options):
+    def ask_for_confirm_with_message(cls, ui, prompt='Do you agree?', message='', **options):
         """Returns True if user agrees, False otherwise"""
-        return cls.get_appropriate_helper().ask_for_confirm_with_message(prompt, message)
+        return cls.get_appropriate_helper(ui).ask_for_confirm_with_message(prompt, message)
 
     @classmethod
     def ask_for_package_list_confirm(cls,
+                                     ui,
                                      prompt='Do you want to install packages?',
                                      package_list=[],
                                      **options):
-        return cls.get_appropriate_helper().ask_for_package_list_confirm(prompt,
-                                                                         package_list,
-                                                                         **options)
+        return cls.get_appropriate_helper(ui).ask_for_package_list_confirm(prompt,
+                                                                           package_list,
+                                                                           **options)
+
+    @classmethod
+    def ask_for_input_with_prompt(cls, ui, prompt='', **options):
+        """Ask user for written input with prompt"""
+        return cls.get_appropriate_helper(ui).ask_for_input_with_prompt(prompt=prompt, **options)
 
 
 @DialogHelper.register_helper
@@ -212,10 +251,7 @@ class CliDialogHelper(object):
     yes_list = ['y', 'yes']
     yesno_list = yes_list + ['n', 'no']
 
-    if sys.version_info[0] < 3:
-        inp = raw_input
-    else:
-        inp = input
+    inp = raw_input if not six.PY3 else input
 
     @classmethod
     def is_available(cls):
@@ -273,10 +309,21 @@ class CliDialogHelper(object):
                 else:
                     print('\n'.join(sorted(package_list)))
 
+    @classmethod
+    def ask_for_input_with_prompt(cls, prompt, **options):
+        msg = options.get('message', None)
+        if msg:
+            print(msg)
+        print('{prt}: '.format(prt=prompt), end='')
+        inp = cls._read_inp()
+        if not inp:
+            print()
+        return inp or ''
+
 
 @DialogHelper.register_helper
 class GtkDialogHelper(object):
-    shortname = 'gtk'
+    shortname = 'gui_gtk+'
     Gtk = None
     Gdk = None
     top_window = None
@@ -315,9 +362,10 @@ class GtkDialogHelper(object):
         return cls.get_gtk().Button(label=label)
 
     @classmethod
-    def _get_pwd_entry(cls):
+    def _get_input_entry(cls, passwd=False):
         entry = cls.get_gtk().Entry()
-        entry.set_visibility(False)
+        if passwd:
+            entry.set_visibility(False)
         return entry
 
     @classmethod
@@ -414,8 +462,8 @@ class GtkDialogHelper(object):
         box = cls._get_gtk_box(win)
         grid = cls._create_gtk_grid(win)
         box.add(grid)
-        yes_btn, no_btn = cls._create_yes_no(win)
-        pwd = cls._get_pwd_entry()
+        yes_btn, no_btn = cls._create_yes_no(win, yes='OK', no='Cancel')
+        pwd = cls._get_input_entry(passwd=True)
 
         grid.attach(pwd, 0, 0, 2, 1)
         grid.attach(no_btn, 0, 1, 1, 1)
@@ -491,3 +539,44 @@ class GtkDialogHelper(object):
         win.run()
         Gdk.threads_leave()
         return win.ok or None
+
+    @classmethod
+    def ask_for_input_with_prompt(cls, prompt, **options):
+        """
+        Show a window asking user for written input
+        """
+        Gtk = cls.get_gtk()
+        Gdk = cls.get_gdk()
+        Gdk.threads_enter()
+        win = Gtk.Dialog(title='Prompt')
+        win.ok = False
+
+        box = cls._get_gtk_box(win)
+        grid = cls._create_gtk_grid(win)
+        box.add(grid)
+        yes_btn, no_btn = cls._create_yes_no(win, yes='OK', no='Cancel')
+        inp = cls._get_input_entry()
+        prt = Gtk.Label(prompt)
+        prt.set_alignment(0, 0)
+        prt.set_max_width_chars(40)
+        prt.set_single_line_mode(False)
+
+        message = options.get('message', None)
+        offset = 0
+        if message:
+            msg = Gtk.Label(message)
+            msg.set_alignment(0, 0)
+            msg.set_max_width_chars(40)
+            msg.set_single_line_mode(False)
+            offset = 1
+            grid.attach(msg, 0, 0, 2, 1)
+
+        grid.attach(prt, 0, 0+offset, 2, 1)
+        grid.attach(inp, 0, 1+offset, 2, 1)
+        grid.attach(no_btn, 0, 2+offset, 1, 1)
+        grid.attach(yes_btn, 1, 2+offset, 1, 1)
+
+        win.show_all()
+        win.run()
+        Gdk.threads_leave()
+        return inp.get_text() if win.ok else None
