@@ -12,8 +12,9 @@ from gi.repository import Gtk, Gdk
 from gi.repository import GLib
 
 from devassistant.bin import TopAssistant
-from devassistant.logger import logger_gui
-from devassistant import current_run, settings
+from devassistant import logger
+from devassistant import settings
+from devassistant.config_manager import config_manager
 
 from devassistant.gui import path_window
 from devassistant.gui import run_window
@@ -29,7 +30,21 @@ class MainWindow(object):
     """
 
     def __init__(self):
-        current_run.UI = 'gtk'
+        # Used for debugging
+        console_handler = logging.StreamHandler(stream=sys.stdout)
+        console_formatter = logging.Formatter('%(asctime)s %(levelname)s - %(message)s')
+        console_handler.setFormatter(console_formatter)
+        console_handler.setLevel(logging.INFO)
+        logger.logger_gui.addHandler(console_handler)
+        # End used for debugging
+        # Setup logger for warnings and errors that can occur before running assistants
+        #  (e.g. errors from loading faulty assistants)
+        # TODO: are the logger.Devassistant* classes good enough or do we need special ones?
+        ch = logger.DevassistantClHandler(stream=sys.stderr)
+        ch.setFormatter(logger.DevassistantClFormatter())
+        ch.setLevel(logging.WARNING)
+        logger.logger.addHandler(ch)
+        # End setup logger
         self.builder = Gtk.Builder()
         self.builder.add_from_file(GLADE_FILE)
         self.main_win = self.builder.get_object("mainWindow")
@@ -42,10 +57,10 @@ class MainWindow(object):
                                                self.builder,
                                                self.gui_helper)
         self.main_handlers = {
-            "on_mainWindow_delete_event": Gtk.main_quit,
+            "on_mainWindow_delete_event": self.delete_event,
             "on_browsePathBtn_clicked": self.path_window.browse_path,
             "on_nextPathBtn_clicked": self.path_window.next_window,
-            "on_pathWindow_delete_event": Gtk.main_quit,
+            "on_pathWindow_delete_event": self.delete_event,
             "on_runWindow_delete_event": self.run_window.delete_event,
             "on_runWindow_destroy": self.run_window.destroy,
             "on_prevPathBtn_clicked": self.path_window.prev_window,
@@ -80,16 +95,10 @@ class MainWindow(object):
         self.notebook.show()
         self.kwargs = dict()
         self.data = dict()
-        # Used for debugging
-        console_handler = logging.StreamHandler(stream=sys.stdout)
-        console_formatter = logging.Formatter('%(asctime)s %(levelname)s - %(message)s')
-        console_handler.setFormatter(console_formatter)
-        console_handler.setLevel(logging.INFO)
-        logger_gui.addHandler(console_handler)
-        # End used for debugging
-        self.data = dict()
         self.dev_assistant_path = []
         self.main_win.show_all()
+        # Load configuration file
+        config_manager.load_configuration_file()
         # Thread should be defined here
         # because of timeout and threads sharing.
         GLib.threads_init()
@@ -128,6 +137,14 @@ class MainWindow(object):
                 # If assistant has more subassistants then create button with menu
                 self.gui_helper.add_submenu(grid_lang, ass, row, column)
             column += 1
+
+        # Install More Assistants button
+        if column > 2:
+            row += 1
+            column = 0
+        self.gui_helper.add_install_button(grid_lang, row, column)
+        column += 1
+
         if row == 0 and len(sub_as) < 3:
             while column < 3:
                 btn = self.gui_helper.create_button(style=Gtk.ReliefStyle.NONE)
@@ -143,6 +160,17 @@ class MainWindow(object):
         """
         self.dev_assistant_path = list(item)
 
+    def _open_path_window(self):
+        """
+        Hides this window and opens path window.
+        Passes all needed data and kwargs.
+        """
+        self.data['top_assistant'] = self.top_assistant
+        self.data['current_main_assistant'] = self.get_current_main_assistant()
+        self.data['kwargs'] = self.kwargs
+        self.path_window.open_window(self.data)
+        self.main_win.hide()
+
     def sub_menu_pressed(self, widget, event):
         """
         Function serves for getting full assistant path and
@@ -154,11 +182,7 @@ class MainWindow(object):
                 del self.kwargs[settings.SUBASSISTANT_N_STRING.format(index)]
             self.kwargs[settings.SUBASSISTANT_N_STRING.format(index)] = data
         self.kwargs['subassistant_0'] = self.get_current_main_assistant().name
-        self.data['top_assistant'] = self.top_assistant
-        self.data['current_main_assistant'] = self.get_current_main_assistant()
-        self.data['kwargs'] = self.kwargs
-        self.path_window.open_window(widget, self.data)
-        self.main_win.hide()
+        self._open_path_window()
 
     def get_current_main_assistant(self):
         """
@@ -176,11 +200,23 @@ class MainWindow(object):
         self.kwargs['subassistant_1'] = data
         if 'subassistant_2' in self.kwargs:
             del self.kwargs['subassistant_2']
-        self.data['top_assistant'] = self.top_assistant
-        self.data['current_main_assistant'] = self.get_current_main_assistant()
-        self.data['kwargs'] = self.kwargs
-        self.path_window.open_window(widget, self.data)
-        self.main_win.hide()
+        self._open_path_window()
+
+    def install_btn_clicked(self, data=None):
+#        self.install_window.open_window()
+        text = 'To install new Assistants from the DevAssistant Package Index, ' \
+               'you must go to the terminal and run "da pkg install [PACKAGE]" ' \
+               '(without the quotation marks), where [PACKAGE] is the name of ' \
+               'the package you want to install.\n\n' \
+               'If you do not know what package you are looking for, use\n"da '\
+               'pkg search [NAME]", or browse the Index at ' \
+               'https://dapi.devassistant.org.\n\n' \
+               'This is a temporary measure, you will be able to install Assistants ' \
+               'from the GUI soon.'
+        dialog = self.gui_helper.create_message_dialog(text, buttons=Gtk.ButtonsType.OK, icon=Gtk.MessageType.WARNING)
+        dialog.run()
+        dialog.destroy()
+
 
     def browse_path(self, window):
         self.path_window.browse_path()
@@ -208,3 +244,7 @@ class MainWindow(object):
                              event.button.button, event.time)
             return True
         return False
+
+    def delete_event(self, widget, event, data=None):
+        config_manager.save_configuration_file()
+        Gtk.main_quit()
