@@ -1,9 +1,10 @@
-from devassistant import command
+from devassistant import lang
 from devassistant.logger import logger
 from devassistant import exceptions
 from devassistant import utils
 from devassistant import yaml_assistant
 
+
 class PathRunner(object):
     def __init__(self, path, override_sys_excepthook=True):
         self.path = path
@@ -21,7 +22,7 @@ class PathRunner(object):
         """
         deps = self.path[-1].dependencies(parsed_args)
 
-        command.Command('dependencies', deps, parsed_args).run()
+        lang.Command('dependencies', deps, parsed_args).run()
 
     def _run_path_run(self, stage, parsed_args):
         """Runs run section with given stage from leaf assistants.
@@ -30,6 +31,13 @@ class PathRunner(object):
         """
         self.path[-1].run(stage, parsed_args)
 
+    def _log_if_not_logged(self, err):
+        if not getattr(err, 'already_logged', False):
+            # this is here primarily because of log_ command, that logs the message itself
+            logger.error(err)
+
+        return err
+
     def run(self, **parsed_args):
         """Runs all errors, dependencies and run methods of all *Assistant objects in self.path.
         Raises:
@@ -37,29 +45,33 @@ class PathRunner(object):
         """
         error = None
         # run 'pre_run', 'logging', 'dependencies' and 'run'
-        try: # serve as a central place for error logging
+        try:  # serve as a central place for error logging
             self._logging(parsed_args)
-            if not 'deps_only' in parsed_args:
+            if 'deps_only' not in parsed_args:
                 self._run_path_run('pre', parsed_args)
             self._run_path_dependencies(parsed_args)
-            if not 'deps_only' in parsed_args:
+            if 'deps_only' not in parsed_args:
                 self._run_path_run('', parsed_args)
         except exceptions.ExecutionException as e:
-            if not getattr(e, 'already_logged', False):
-                # this is here primarily because of log_ command, that logs the message itself
-                logger.error(utils.u(e))
-            error = e
+            error = self._log_if_not_logged(e)
+            if isinstance(e, exceptions.YamlError):  # if there's a yaml error, just shut down
+                raise e
 
         # in any case, run post_run
-        try: # serve as a central place for error logging
+        try:  # serve as a central place for error logging
             self._run_path_run('post', parsed_args)
         except exceptions.ExecutionException as e:
-            if not getattr(e, 'already_logged', False):
-                # this is here primarily because of log_ command, that logs the message itself
-                logger.error(utils.u(e))
-            error = e
+            error = self._log_if_not_logged(e)
 
-        if error: raise error
+        # exitfuncs are run all regardless of exceptions; if there is an exception in one
+        #  of them, this function will raise it at the end
+        try:
+            utils.run_exitfuncs()
+        except exceptions.ExecutionException as e:
+            error = self._log_if_not_logged(e)
+
+        if error:
+            raise error
 
     def stop(self):
         for a in self.path:
