@@ -1,11 +1,13 @@
 import os
 
 from devassistant import cache
-from devassistant import current_run
+from devassistant import exceptions
 from devassistant.logger import logger
 from devassistant import yaml_loader
 from devassistant import settings
 from devassistant import yaml_assistant
+from devassistant import yaml_checker
+
 
 class YamlAssistantLoader(object):
     assistants_dirs = list(map(lambda x: os.path.join(x, 'assistants'), settings.DATA_DIRECTORIES))
@@ -46,8 +48,8 @@ class YamlAssistantLoader(object):
             dirs = [os.path.join(d, tl) for d in cls.assistants_dirs]
             file_hierarchy = cls.get_assistants_file_hierarchy(dirs)
             # load all if we're not using cache or if we fail to load it
-            load_all = not current_run.USE_CACHE
-            if current_run.USE_CACHE:
+            load_all = not settings.USE_CACHE
+            if settings.USE_CACHE:
                 try:
                     cch = cache.Cache()
                     cch.refresh_role(tl, file_hierarchy)
@@ -55,9 +57,10 @@ class YamlAssistantLoader(object):
                                                                                   superas_dict[tl],
                                                                                   role=tl)
                 except BaseException as e:
-                    logger.debug(e)
+                    logger.debug('Failed to use DevAssistant cachefile {0}: {1}'.format(
+                        settings.CACHE_FILE, e))
                     load_all = True
-            if load_all or not current_run.USE_CACHE:
+            if load_all:
                 cls._assistants[tl] = cls.get_assistants_from_file_hierarchy(file_hierarchy,
                                                                              superas_dict[tl],
                                                                              role=tl)
@@ -107,19 +110,20 @@ class YamlAssistantLoader(object):
             references to instances of their subassistants (and their subassistants, ...)
         """
         result = []
-        warn_msg ='Failed to load assistant {source}, skipping subassistants.'
+        warn_msg = 'Failed to load assistant {source}, skipping subassistants.'
 
         for name, attrs in file_hierarchy.items():
             loaded_yaml = yaml_loader.YamlLoader.load_yaml_by_path(attrs['source'])
-            if not loaded_yaml: # there was an error parsing yaml
+            if loaded_yaml is None:  # there was an error parsing yaml
                 logger.warning(warn_msg.format(source=attrs['source']))
                 continue
-            ass = cls.assistant_from_yaml(attrs['source'],
-                                          loaded_yaml,
-                                          superassistant,
-                                          role=role)
-            if not ass: # assistant isn't written properly (misses top-level mapping)
-                logger.warning(warn_msg.format(source=attrs['source']))
+            try:
+                ass = cls.assistant_from_yaml(attrs['source'],
+                                              loaded_yaml,
+                                              superassistant,
+                                              role=role)
+            except exceptions.YamlError as e:
+                logger.warning(e)
                 continue
             ass._subassistants = cls.get_assistants_from_file_hierarchy(attrs['subhierarchy'],
                                                                         ass,
@@ -145,9 +149,11 @@ class YamlAssistantLoader(object):
         Returns:
             hierarchy structure that looks like this:
             {'assistant1':
-                {'source': '/path/to/assistant1.yaml', 'subhierarchy': {<hierarchy of subassistants>}},
+                {'source': '/path/to/assistant1.yaml',
+                 'subhierarchy': {<hierarchy of subassistants>}},
              'assistant2':
-                {'source': '/path/to/assistant2.yaml', 'subhierarchy': {<another hieararchy of subassistants}}
+                {'source': '/path/to/assistant2.yaml',
+                 'subhierarchy': {<another hierarchy of subassistants}}
             }
         """
         result = {}
@@ -157,7 +163,8 @@ class YamlAssistantLoader(object):
                 if assistant_name not in result:
                     subas_dirs = [os.path.join(dr, assistant_name) for dr in dirs]
                     result[assistant_name] = {'source': os.path.join(d, f),
-                                              'subhierarchy': cls.get_assistants_file_hierarchy(subas_dirs)}
+                                              'subhierarchy':
+                                              cls.get_assistants_file_hierarchy(subas_dirs)}
 
         return result
 
@@ -173,18 +180,15 @@ class YamlAssistantLoader(object):
             superassistant: superassistant of this assistant
         Returns:
             YamlAssistant instance constructed from y with source file source
+        Raises:
+            YamlError: if the assistant is malformed
         """
-        try:
-            name, attrs = y.popitem() # assume only one key and value
-        except AttributeError:
-            logger.warning('File {source} is not formatted properly - no top level mapping.'.\
-                    format(source=source))
-            return None
-        assistant = yaml_assistant.YamlAssistant(name,
-                                                 attrs,
-                                                 source,
-                                                 superassistant,
-                                                 fully_loaded=fully_loaded,
-                                                 role=role)
+        # In pre-0.9.0, we required assistant to be a mapping of {name: assistant_attributes}
+        # now we allow that, but we also allow omitting the assistant name and putting
+        # the attributes to top_level, too.
+        name = os.path.splitext(os.path.basename(source))[0]
+        yaml_checker.check(source, y)
+        assistant = yaml_assistant.YamlAssistant(name, y, source, superassistant,
+            fully_loaded=fully_loaded, role=role)
 
         return assistant
