@@ -2,13 +2,18 @@ import os
 import shutil
 import time
 
+import pytest
 import yaml
 
+import devassistant
+
 from devassistant.cache import Cache
+from devassistant.exceptions import YamlTypeError
 from devassistant import settings
-from devassistant.version import VERSION
 from devassistant.yaml_assistant_loader import YamlAssistantLoader
 
+from test.logger import TestLoggingHandler
+
 # the paths in this dicts are truncated to make tests pass in any location
 # (if not truncated, they contain e.g. home dir on your machine, etc.)
 # therefore we need a special comparison method in class below
@@ -26,8 +31,8 @@ correct_cache = \
                                                              'python': {'flags': ['mock'],
                                                                         'help': 'Display python version',
                                                                         'nargs': '?'},
-                                                             'some_arg': {'flags': ['-s',
-                                                                                    '--some-arg']}},
+                                                             'some_arg': {'flags': ['-s', '--some-arg'],
+                                                                          'gui_hints': {'default': '$(whoami)'}}},
                                                     'fullname': 'D Language Tool'},
                                           'snippets': {'snippet1': 'dontcheck'},
                                           'ctime': 'dontcheck',
@@ -56,10 +61,10 @@ correct_cache = \
                                           'ctime': 'dontcheck',
                                           'source': 'test/fixtures/assistants/crt/f/g.yaml',
                                           'subhierarchy': {}}}}},
- 'mod': {},
+ 'twk': {},
  'prep': {},
- 'task': {},
- 'version': VERSION}
+ 'extra': {},
+ 'version': devassistant.__version__}
 
 class TestCache(object):
     cf = settings.CACHE_FILE
@@ -69,6 +74,7 @@ class TestCache(object):
         if os.path.exists(self.cf):
             os.unlink(self.cf)
         self.cch = Cache()
+        self.tlh = TestLoggingHandler.create_fresh_handler()
 
     def teardown_method(self, method):
         while self.remove_files:
@@ -99,7 +105,7 @@ class TestCache(object):
         os.utime(self.datafile_path(path), None)
 
     def assert_cache_newer(self, path):
-        assert os.path.getctime(self.cch.cache_file) > os.path.getctime(self.datafile_path(path))
+        assert os.path.getctime(self.cch.cache_file) >= os.path.getctime(self.datafile_path(path))
 
     def assert_cache_content(self, expected, actual):
         assert len(expected) == len(actual)
@@ -164,7 +170,7 @@ class TestCache(object):
         self.addme_copy('addme_change_fullname.yaml', 'assistants/crt/addme.yaml')
         self.create_or_refresh_cache()
         assert addme['attrs']['fullname'] == 'Fullname changed!'
-        
+
         # change current snippet (will change argument flag)
         # snippets are cached during one startup => reset snippet cache manually
         # TODO: fix this ^^
@@ -196,8 +202,16 @@ class TestCache(object):
         assert prev_time < os.path.getctime(self.cch.cache_file)
 
     def test_cache_stays_if_same_version(self):
-        self.create_fake_cache({'version': VERSION})
+        self.create_fake_cache({'version': devassistant.__version__})
         prev_time = os.path.getctime(self.cch.cache_file)
         time.sleep(0.1)
         Cache()
         assert prev_time == os.path.getctime(self.cch.cache_file)
+
+    def test_cache_doesnt_log_higher_than_debug(self):
+        # make sure that there's an assistant with an error
+        with pytest.raises(YamlTypeError):
+            self.create_or_refresh_cache(assistants='assistants_malformed')
+        # make sure that there are only DEBUG messages logged
+        for msg in self.tlh.msgs:
+            assert msg[0] == 'DEBUG'
