@@ -1,14 +1,21 @@
+# -*- coding: utf8 -*-
+import copy
 import os
+import sys
 
 import pytest
 from flexmock import flexmock
+import yaml
 
 from devassistant import lang
 from devassistant.assistant_base import AssistantBase
-from devassistant.command_helpers import DialogHelper
+from devassistant.command_helpers import ClHelper, DialogHelper
 from devassistant.command_runners import AskCommandRunner, ClCommandRunner, \
     Jinja2Runner, LogCommandRunner, NormalizeCommandRunner, UseCommandRunner, \
-    SetupProjectDirCommandRunner
+    SetupProjectDirCommandRunner, PingPongCommandRunner, LoadCmdCommandRunner, \
+    EnvCommandRunner
+# also import just command_runners to have access to command_runners.command_runners
+from devassistant import command_runners
 from devassistant.exceptions import CommandException, RunException
 from devassistant.lang import Command
 from devassistant.yaml_assistant import YamlAssistant
@@ -32,17 +39,27 @@ class TestAskCommandRunner(object):
     def test_run_password(self):
         flexmock(DialogHelper)
         DialogHelper.should_receive('ask_for_password').and_return('foobar')
-        comm = Command('ask_password', {})
+        comm = Command('ask_password', {}, {'__ui__': 'cli'})
         res = self.acr.run(comm)
 
         assert res[0] is True
         assert res[1] == 'foobar'
 
+    @pytest.mark.parametrize('inp', ['', 'foo', u'foo'])
+    def test_run_input(self, inp):
+        flexmock(DialogHelper)
+        DialogHelper.should_receive('ask_for_input_with_prompt').and_return(inp)
+        comm = Command('ask_input', {}, {'__ui__': 'cli'})
+        res = self.acr.run(comm)
+
+        assert res[0] is bool(inp)
+        assert res[1] == inp
+
     @pytest.mark.parametrize('decision', [True, False])
     def test_run_confirm(self, decision):
         flexmock(DialogHelper)
         DialogHelper.should_receive('ask_for_confirm_with_message').and_return(decision)
-        comm = Command('ask_confirm', {})
+        comm = Command('ask_confirm', {}, {'__ui__': 'cli'})
         res = self.acr.run(comm)
 
         assert res[0] is decision
@@ -61,6 +78,7 @@ class TestUseCommandRunner(object):
 
     def setup_method(self, method):
         self.ccr = UseCommandRunner
+        self.tlh = TestLoggingHandler.create_fresh_handler()
 
     def test_matches(self):
         assert self.ccr.matches(Command('use', None))
@@ -136,6 +154,27 @@ class TestUseCommandRunner(object):
 
         assert result == expected
 
+    def test_run_with_specified_args(self):
+        kwargs = {'spam': 'something',
+                  'nospam': 'hahaha',
+                  '__magic_val__': 'it\'s magical',
+                  '__assistant__': self.ass['leaf']}
+
+        com = Command('use', {'sect': 'self.run',
+                              'args': {'spam': 'something else', 'spam2': 'blah'}}, kwargs)
+
+        class Matcher(object):
+            def __eq__(self, other):
+                assert len(other) == 4
+                assert other['spam'] == 'something else'
+                assert other['spam2'] == 'blah'
+                assert other['__magic_val__'] == 'it\'s magical'
+                assert '__assistant__' in other
+                return True
+
+        flexmock(lang).should_receive('run_section').with_args(list, Matcher())
+        com.run()
+
 
 class TestClCommandRunner(object):
     def setup_method(self, method):
@@ -159,13 +198,29 @@ class TestClCommandRunner(object):
         self.cl.run(Command('cl_i', 'id'))
         assert ('INFO', 'id') in self.tlh.msgs
 
+    def test_p_flag_pasesses_even_if_subcommand_fails(self):
+        self.cl.run(Command('cl_ip', 'false'))
+        assert ('INFO', 'false') in self.tlh.msgs
+
+    def test_passes_env(self):
+        self.cl.run(Command('cl_i', 'echo $DEVASSISTANTTESTFOO',
+            kwargs={'__env__': {'DEVASSISTANTTESTFOO': 'foo'}}))
+        assert ('INFO', 'foo') in self.tlh.msgs
+
 
 class TestDependenciesCommandRunner(object):
     pass
 
 
 class TestDotDevassistantCommandRunner(object):
-    pass
+    def test_dda_w_takes_section_as_literal(self, tmpdir):
+        # we want to make sure that the section we write is not evaluated, not even substituted
+        kwargs = {'foo': 'bar'}
+        to_write = [tmpdir.strpath, {'run': [{'$cwd~': '$(pwd)'}, {'log_i': '$foo'}]}]
+        open(os.path.join(tmpdir.strpath, '.devassistant'), 'w').close()
+        Command('dda_w', to_write, kwargs).run()
+        content = open(os.path.join(tmpdir.strpath, '.devassistant')).read()
+        assert content == yaml.dump(to_write[1], default_flow_style=False)
 
 
 class TestGitHubCommandRunner(object):
@@ -288,19 +343,42 @@ class TestNormalizeCommandRunner(object):
         bad_string = '-+\\|()[]{}<>,./:\'" \t;`!@#$%^&*'
         self.n.run(Command('normalize', bad_string)) == (True, '_' * len(bad_string))
 
+    def test_unicode_chars(self):
+        s = 'ěšč'
+        assert self.n.run(Command('normalize', s)) == (True, 'esc')
+        s = u'ěšč'
+        assert self.n.run(Command('normalize', s)) == (True, 'esc')
+
+    def test_ok_chars(self):
+        i = {'what': 'foo-bar.+*', 'ok_chars': '-.'}
+        assert self.n.run(Command('normalize', i)) == (True, 'foo-bar.__')
+
 
 class TestSCLCommandRunner(object):
     def setup_method(self, method):
         self.tlh = TestLoggingHandler.create_fresh_handler()
 
-    def test_scl_passes_scls_list_to_command_invocation(self):
-        # please don't use $__scls__ in actual assistants :)
-        # scl runner has to use the unformatted input
-        inp = [{'log_i': '$__scls__'}]
-        fmtd_inp = [{'log_i': 'this should not be used'}]
-        c = Command('scl enable foo bar', inp)
-        c.run()
-        assert ('INFO', "[['enable', 'foo', 'bar']]") in self.tlh.msgs
+    def test_scl_with_nested_calls(self):
+        # https://github.com/bkabrda/devassistant/issues/234
+        # tests proper nesting of SCL commands and also elimination of identical calls
+        inp = [{'scl enable hamham foo': [{'scl enable spamspam': [{'cl': 'ls'}]}]}]
+        c = Command('scl enable spamspam', inp)
+        with pytest.raises(RunException):
+            c.run()
+        # make sure to remove scl command processors from ClHelper
+        ClHelper.command_processors = {}
+        res_lines = [['scl enable spamspam - << DA_SCL_enable_spamspam_EOF',
+                      'scl enable hamham foo - << DA_SCL_enable_hamham_foo_EOF',
+                      'ls',
+                      'DA_SCL_enable_hamham_foo_EOF',
+                      'DA_SCL_enable_spamspam_EOF']]
+        # shuffle first two and last two lines to get possible permutations of scl
+        #  call order (command processors are in dict, which has arbitrary order)
+        res_lines.append(copy.deepcopy(res_lines[0]))
+        res_lines[1][0], res_lines[1][1] = res_lines[1][1], res_lines[1][0]
+        res_lines[1][3], res_lines[1][4] = res_lines[1][4], res_lines[1][3]
+        assert ('DEBUG', '\n'.join(res_lines[0])) in self.tlh.msgs or \
+               ('DEBUG', '\n'.join(res_lines[1])) in self.tlh.msgs
 
 
 class TestSetupProjectDirCommandRunner(object):
@@ -366,3 +444,128 @@ class TestSetupProjectDirCommandRunner(object):
             assert kwargs[varnames[0]] == (os.path.dirname(path) or '.')
             assert kwargs[varnames[1]] == os.path.basename(path)
             assert kwargs[varnames[2]] == normalized
+
+
+class TestPingPongCommandRunner(object):
+    fixtures = os.path.join(os.path.dirname(__file__), 'fixtures', 'pingpong')
+
+    def setup_method(self, method):
+        self.tlh = TestLoggingHandler.create_fresh_handler()
+
+    def _run_pingpong(self, script, ctxt):
+        return Command('pingpong',
+            '{0} {1}'.format(sys.executable, os.path.join(self.fixtures, script)),
+            ctxt).run()
+
+    def test_ok_run(self):
+        ctxt = {'foo': 'bar'}
+        res = self._run_pingpong('ok_run.py', ctxt)
+
+        assert res == (True, 'Everything OK')
+        assert 'foo' not in ctxt
+        assert ctxt['set_by_pp_client'] == 'bar'
+
+        assert ('INFO', 'from var: bar') in self.tlh.msgs
+        assert ('INFO', 'from ctxt: bar') in self.tlh.msgs
+        assert ('DEBUG', 'foo') in self.tlh.msgs
+
+    def test_no_return(self):
+        with pytest.raises(CommandException) as e:
+            self._run_pingpong('no_return.py', {})
+        assert 'PingPong run method ended with unexpected result: None (expected 2-tuple)' in \
+            str(e.value)
+
+    def test_tracebacks(self):
+        with pytest.raises(CommandException) as e:
+            self._run_pingpong('tracebacks.py', {})
+        assert 'PingPong run method ended with an exception:' in str(e.value)
+        assert 'BaseException: problem' in str(e.value)
+
+
+class TestLoadCmdCommandRunner(object):
+    fixtures = os.path.join(os.path.dirname(__file__), 'fixtures', 'files', 'crt', 'commands')
+
+    def setup_method(self, method):
+        self.old_command_runners = command_runners.command_runners
+        command_runners.command_runners = copy.deepcopy(self.old_command_runners)
+        self.tlh = TestLoggingHandler.create_fresh_handler()
+
+    def teardown_method(self, method):
+        command_runners.command_runners = self.old_command_runners
+
+    def _test_loaded_commands_work(self, prefix='', which=['CR1', 'CR2']):
+        if 'CR1' in which:
+            r1 = Command(prefix + 'barbarbar', 'foo').run()
+            assert ('INFO', 'CR1: Doing something ...') in self.tlh.msgs
+            assert r1 == (True, 'foobar')
+        else:
+            with pytest.raises(CommandException):
+                Command(prefix + 'barbarbar', 'foo').run()
+
+        if 'CR2' in which:
+            r2 = Command(prefix + 'spamspamspam', 'foo').run()
+            assert ('INFO', 'CR2: Doing something ...') in self.tlh.msgs
+            assert r2 == (True, 'foospam')
+        else:
+            with pytest.raises(CommandException):
+                Command(prefix + 'spamspamspam', 'foo').run()
+
+    def test_basic(self):
+        r = Command('load_cmd', {'source': 'a.py'},
+                    {'__files_dir__': [self.fixtures]}).run()
+        assert r[0] == True
+        assert set(r[1]) == set(['CR1', 'CR2'])
+
+        self._test_loaded_commands_work()
+
+    def test_specified_by_string(self):
+        r = Command('load_cmd', 'crt/commands/a.py').run()
+        assert r[0] == True
+        assert set(r[1]) == set(['CR1', 'CR2'])
+
+        self._test_loaded_commands_work()
+
+    def test_load_only(self):
+        r = Command('load_cmd', {'from_file': {'source': 'a.py'}, 'load_only': 'CR1'},
+                    {'__files_dir__': [self.fixtures]}).run()
+        assert r == (True, ['CR1'])
+
+        self._test_loaded_commands_work(which=['CR1'])
+
+    def test_prefix(self):
+        r = Command('load_cmd', {'from_file': 'crt/commands/a.py', 'prefix': 'prefix'}).run()
+        assert r[0] == True
+        assert set(r[1]) == set(['CR1', 'CR2'])
+
+        self._test_loaded_commands_work(prefix='prefix.')
+        # also test that they're not loaded in the '' prefix
+        self._test_loaded_commands_work(which=[])
+
+    def test_overrides_core_runners(self):
+        Command('load_cmd', 'crt/commands/log.py').run()
+        r = Command('log_i', 'message').run()
+
+        assert ('INFO', 'Got you!') in self.tlh.msgs
+        assert ('INFO', 'message') not in self.tlh.msgs
+        assert r == (True, 'heee heee')
+
+
+class TestEnvCommandRunner(object):
+    def setup_method(self, method):
+        self.kwargs = {'__env__': {'foo': 'bar', 'spam': 'spam'}}
+
+    def test_set(self):
+        res = EnvCommandRunner.run(Command('env_set', {'foo': 'changed', 'some': 'value'},
+            self.kwargs))
+        assert res == (True, {'foo': 'changed', 'some': 'value'})
+        assert self.kwargs == {'__env__': {'foo': 'changed', 'some': 'value', 'spam': 'spam'}}
+
+    def test_unset_one(self):
+        res = EnvCommandRunner.run(Command('env_unset', 'spam', self.kwargs))
+        assert res == (True, {'spam': 'spam'})
+        assert self.kwargs == {'__env__': {'foo': 'bar'}}
+
+    def test_unset_more(self):
+        res = EnvCommandRunner.run(Command('env_unset', ['foo', 'spam'], self.kwargs))
+        assert res == (True, {'foo': 'bar', 'spam': 'spam'})
+        assert self.kwargs == {'__env__': {}}
