@@ -4,7 +4,8 @@ import pytest
 import yaml
 
 from devassistant.assistant_base import AssistantBase
-from devassistant import current_run
+from devassistant import exceptions
+from devassistant import settings
 from devassistant.yaml_assistant_loader import YamlAssistantLoader
 
 from test.logger import TestLoggingHandler
@@ -21,6 +22,7 @@ class TestYamlAssistantLoader(object):
     def teardown_method(self, method):
         # in case that a test changed the dirs
         self.reset_yl_assistants_dirs()
+        settings.USE_CACHE = True
 
     def reset_yl_assistants_dirs(self):
         self.yl.assistants_dirs = [os.path.join(os.path.dirname(__file__), 'fixtures', 'assistants')]
@@ -35,7 +37,7 @@ class TestYamlAssistantLoader(object):
         # TODO: probably move testing to yaml_assistants tests, since
         # most of this stuff is done there now (although it's testable here)
         y = self.load_yaml_from_fixture('c')
-        a = self.yl.assistant_from_yaml('foo', y, CreatorAssistant())
+        a = self.yl.assistant_from_yaml('c.yaml', y, CreatorAssistant())
 
         assert a.name == 'c'
         assert a.fullname == 'C Language Tool'
@@ -73,19 +75,25 @@ class TestYamlAssistantLoader(object):
                                                 'fixtures',
                                                 'assistants_with_snippet_problems')]
         y = self.load_yaml_from_fixture('no_snippet_for_arg')
-        klass = self.yl.assistant_from_yaml('foo', y, CreatorAssistant())
-        assert ('WARNING', 'Couldn\'t expand argument bar in assistant no_snippet_for_arg: no such snippet: doesnt_exist') in self.tlh.msgs
+        self.yl.assistant_from_yaml('no_snippet_for_arg.yaml', y, CreatorAssistant())
+        assert ('WARNING',
+                'Problem when constructing argument bar in assistant no_snippet_for_arg: ' + \
+                'no such snippet: doesnt_exist') \
+            in self.tlh.msgs
 
     def test_assistant_from_yaml_doesnt_fail_on_missing_arg(self):
         self.yl.assistants_dirs = [os.path.join(os.path.dirname(__file__),
                                                 'fixtures',
                                                 'assistants_with_snippet_problems')]
         y = self.load_yaml_from_fixture('no_arg_in_snippet')
-        a = self.yl.assistant_from_yaml('foo', y, CreatorAssistant())
-        assert ('WARNING', 'Couldn\'t find argument bar in snippet snippet1 wanted by assistant no_arg_in_snippet.') in self.tlh.msgs
+        self.yl.assistant_from_yaml('no_arg_in_snippet.yaml', y, CreatorAssistant())
+        assert ('WARNING',
+                'Problem when constructing argument bar in assistant no_arg_in_snippet: ' + \
+                'Couldn\'t find arg bar in snippet snippet1.') \
+            in self.tlh.msgs
 
     def test_no_cache(self):
-        current_run.USE_CACHE = False
+        settings.USE_CACHE = False
         oldm = self.yl.get_assistants_from_cache_hierarchy
         self.yl.get_assistants_from_cache_hierarchy = None
         # if YamlAssistantLoader tries to load from cache now, it will log debug message
@@ -95,6 +103,15 @@ class TestYamlAssistantLoader(object):
         assert set(['c', 'f']) == set(map(lambda x: x.name, ass))
         self.yl.get_assistants_from_cache_hierarchy = oldm
 
+    def test_get_assistants_from_file_hierarchy_with_empty_assistant(self):
+        empty = os.path.join(os.path.dirname(__file__),
+                             'fixtures',
+                             'empty.yaml')
+        res = self.yl.get_assistants_from_file_hierarchy({'empty': {'source': empty,
+                                                                    'subhierarchy': {}}},
+                                                         None)
+        assert len(res) == 1
+
     def test_get_assistants_from_file_hierarchy_with_bad_syntax(self):
         bad_syntax = os.path.join(os.path.dirname(__file__),
                                   'fixtures',
@@ -115,7 +132,8 @@ class TestYamlAssistantLoader(object):
                                   'assistants_malformed',
                                   'crt',
                                   'a2.yaml')
-        err = 'Failed to load assistant {src}, skipping subassistants.'.format(src=bad_syntax)
+        err = 'In {src}:\nAssistants and snippets must be Yaml mappings, not "asd"!'.\
+            format(src=bad_syntax)
         self.yl.assistants_dirs = [os.path.join(os.path.split(bad_syntax[:-2]))]
         res = self.yl.get_assistants_from_file_hierarchy({'a1': {'source': bad_syntax,
                                                                  'subhierarchy': {}}},
@@ -123,8 +141,6 @@ class TestYamlAssistantLoader(object):
         assert ('WARNING', err) in self.tlh.msgs
         assert res == []
 
-    def test_assistant_from_yaml_logs_and_returns_None_on_bad_assistant(self):
-        res = self.yl.assistant_from_yaml('/foo/bar', 'not a mapping', None)
-        assert ('WARNING', 'File /foo/bar is not formatted properly - no top level mapping.') \
-                in self.tlh.msgs
-        assert res == None
+    def test_assistant_from_yaml_raises_on_bad_assistant(self):
+        with pytest.raises(exceptions.YamlError):
+            self.yl.assistant_from_yaml('/foo/bar', 'not a mapping', None)
